//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "FormComPort.h"
#include "ComPort.h"
#include "ComPortHandle.h"
#include "Log.h"
#include "Settings.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TfrmComPort *frmComPort;
//---------------------------------------------------------------------------
__fastcall TfrmComPort::TfrmComPort(TComponent* Owner)
	: TForm(Owner)
{
}
//---------------------------------------------------------------------------

void __fastcall TfrmComPort::FormShow(TObject *Sender)
{
	RefreshPortList();
	edSerialPortBaudrate->Text = appSettings.SerialPort.baud;
	//chbSerialPortOpenAtStartup->Checked = tmpSettings.SerialPort.openAtStartup;	
}
//---------------------------------------------------------------------------

void TfrmComPort::RefreshPortList(void)
{
	{
		// serial ports list
		std::vector<struct ComPort::S_COM_PORT> ports;
		if(ComPort::EnumeratePortsAlt()) {
			LOG("Error enumerating ports");
		}
		ports = ComPort::GetPorts();
		cbSerialPorts->Items->Clear();
		for(unsigned int i=0; i<ports.size(); i++) {
			cbSerialPorts->Items->Add(ports[i].name.c_str());
		}
		if(ports.size()) {
			cbSerialPorts->ItemIndex = 0;
		} else {
			cbSerialPorts->ItemIndex = -1;
		}
	}

	cbSerialPorts->Text = appSettings.SerialPort.name;
}

int TfrmComPort::PortOpen(AnsiString name, unsigned int baudrate)
{
	if (comPortHandle != INVALID_HANDLE_VALUE)
	{
		LOG("Port already open!\n");
		return -1;
	}

	DCB dcb;

	comPortHandle = CreateFile(name.c_str(), GENERIC_READ | GENERIC_WRITE, 0, NULL, OPEN_EXISTING, 0, NULL);

	if (comPortHandle == INVALID_HANDLE_VALUE)
	{
		LOG("COM port open failed\n");
		return -2;
	}

	enum { OUT_QUEUE_SIZE =	8192 };
	enum { IN_QUEUE_SIZE = 8192 };

	SetupComm(comPortHandle, IN_QUEUE_SIZE, OUT_QUEUE_SIZE);

	dcb.DCBlength = sizeof(dcb);

	GetCommState(comPortHandle, &dcb);

	dcb.BaudRate = baudrate;
	dcb.fParity = TRUE;
	dcb.Parity = NOPARITY;
	dcb.StopBits = ONESTOPBIT;
	dcb.ByteSize = 8;

	int hw_ctrl = 0;

	if (hw_ctrl == 0)
	{
		dcb.fDtrControl = DTR_CONTROL_DISABLE;
		//dcb.fDtrControl = DTR_CONTROL_ENABLE;
		//dcb.fRtsControl = RTS_CONTROL_DISABLE;
		dcb.fRtsControl = RTS_CONTROL_ENABLE;

		dcb.fOutxCtsFlow = FALSE;
		dcb.fOutxDsrFlow = FALSE;
		dcb.fDsrSensitivity = FALSE;
		dcb.fAbortOnError = FALSE;
		dcb.fOutX = FALSE;
		dcb.fInX = FALSE;
		dcb.fErrorChar = FALSE;
		dcb.fNull = FALSE;
	}
	else
	{
		dcb.fDtrControl = DTR_CONTROL_DISABLE;
//		dcb.fRtsControl = RTS_CONTROL_HANDSHAKE;
		dcb.fRtsControl = RTS_CONTROL_TOGGLE;

		dcb.fOutxCtsFlow = TRUE;
		dcb.fOutxDsrFlow = FALSE;
		dcb.fDsrSensitivity = FALSE;
		dcb.fAbortOnError = FALSE;
		dcb.fOutX = FALSE;
		dcb.fInX = FALSE;
		dcb.fErrorChar = FALSE;
		dcb.fNull = FALSE;
	}
	

	SetCommState(comPortHandle, &dcb);

	COMMTIMEOUTS ComTimeouts;
	GetCommTimeouts(comPortHandle, &ComTimeouts);

	ComTimeouts.ReadIntervalTimeout = 1;
	ComTimeouts.ReadTotalTimeoutMultiplier = 1;
	ComTimeouts.ReadTotalTimeoutConstant = 2;
	ComTimeouts.WriteTotalTimeoutMultiplier = 500;
	ComTimeouts.WriteTotalTimeoutConstant = 1000;

	SetCommTimeouts(comPortHandle, &ComTimeouts);

	PurgeComm(comPortHandle, PURGE_TXABORT | PURGE_RXABORT | PURGE_TXCLEAR | PURGE_RXCLEAR);

	LOG("Port %s opened and configured\n", name.c_str());

	return 0;
}

void TfrmComPort::PortClose(void)
{
	if (comPortHandle != INVALID_HANDLE_VALUE)
	{
		CloseHandle(comPortHandle);
		comPortHandle = INVALID_HANDLE_VALUE;
	}
}
void __fastcall TfrmComPort::btnPortOpenClick(TObject *Sender)
{
	int status = PortOpen(appSettings.SerialPort.name, appSettings.SerialPort.baud);
	if (status == 0)
	{
		btnPortOpen->Enabled = false;
		btnPortClose->Enabled = true;
	}
	else
	{
        Application->MessageBox("Error opening port", this->Caption.c_str(), MB_ICONEXCLAMATION);		
	}	
}
//---------------------------------------------------------------------------
void __fastcall TfrmComPort::cbSerialPortsChange(TObject *Sender)
{
	appSettings.SerialPort.name = cbSerialPorts->Text;	
}
//---------------------------------------------------------------------------
void __fastcall TfrmComPort::edSerialPortBaudrateChange(TObject *Sender)
{
	appSettings.SerialPort.baud = StrToIntDef(edSerialPortBaudrate->Text, 115200);	
}
//---------------------------------------------------------------------------
void __fastcall TfrmComPort::btnPortCloseClick(TObject *Sender)
{
	PortClose();
	btnPortOpen->Enabled = true;
	btnPortClose->Enabled = false;	
}
//---------------------------------------------------------------------------
void __fastcall TfrmComPort::btnRefreshListClick(TObject *Sender)
{
	RefreshPortList();	
}
//---------------------------------------------------------------------------

