//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "FormUartMtTest.h"
#include "AppStatus.h"
#include "Settings.h"
#include "TabManager.h"
#include "BtnController.h"
#include "Log.h"
#include "Settings.h"
#include "ComPort.h"
#include "ComPortHandle.h"
#include "common/StaticCheck.h"
#include <stdio.h>
#include <time.h>
#include <Clipbrd.hpp>
#include <vector>
#include <algorithm>

//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TfrmUartMtTest *frmUartMtTest;
//---------------------------------------------------------------------------

namespace {

}	// namespace (anonymous)


__fastcall TfrmUartMtTest::TfrmUartMtTest(TComponent* Owner)
	: TForm(Owner),
	mismatchCount(0),
	totalTxCnt(0)
{
	TabManager::Instance().Register(this);
	Reset();
}
//---------------------------------------------------------------------------

__fastcall TfrmUartMtTest::~TfrmUartMtTest() {

}


void __fastcall TfrmUartMtTest::FormShow(TObject *Sender)
{
	edDataCountMin->Text = appSettings.UartMtTest.dataCountMin;
	edDataCountMax->Text = appSettings.UartMtTest.dataCountMax;
	edIntervalMin->Text = appSettings.UartMtTest.intervalMin;
	edIntervalMax->Text = appSettings.UartMtTest.intervalMax;		
}
//---------------------------------------------------------------------------


void __fastcall TfrmUartMtTest::btnStartTestClick(TObject *Sender)
{
	if (comPortHandle == INVALID_HANDLE_VALUE)
	{
        MessageBox(this->Handle, "Serial port is not opened!", this->Caption.c_str(), MB_ICONEXCLAMATION);
		return;
	}

	tmrTest->Interval = 100;
	tmrTest->Enabled = true;
	btnStartTest->Enabled = false;
	btnStopTest->Enabled = true;
}
//---------------------------------------------------------------------------

void __fastcall TfrmUartMtTest::btnStopTestClick(TObject *Sender)
{
	tmrTest->Enabled = false;
	btnStartTest->Enabled = true;
	btnStopTest->Enabled = false;
}
//---------------------------------------------------------------------------

void __fastcall TfrmUartMtTest::tmrTestTimer(TObject *Sender)
{
	tmrTest->Enabled = false;
	HandleTest();
	unsigned int delta = appSettings.UartMtTest.intervalMax - appSettings.UartMtTest.intervalMin;
	if (delta == 0)
	{
		tmrTest->Interval = appSettings.UartMtTest.intervalMin;
	}
	else
	{
		tmrTest->Interval = appSettings.UartMtTest.intervalMin + (rand() % delta);
	}
	tmrTest->Enabled = true;		
}
//---------------------------------------------------------------------------

void TfrmUartMtTest::HandleTest(void)
{
	unsigned int dataCnt; // to be sent this cycle
	if (appSettings.UartMtTest.dataCountMin == appSettings.UartMtTest.dataCountMax)
	{
		dataCnt = appSettings.UartMtTest.dataCountMax;
	}
	else
	{
		unsigned int randPart = mt_rand_get_u32(&mtSize) % (appSettings.UartMtTest.dataCountMax - appSettings.UartMtTest.dataCountMin + 1);
		dataCnt = appSettings.UartMtTest.dataCountMin + randPart;
	}
	unsigned char txBuffer[Settings::_UartMtTest::MAX_TX_CNT];
	memset(txBuffer, 0x77, sizeof(txBuffer));
	unsigned int txBufferPos = 0;
	unsigned int left = static_cast<unsigned int>(mtTxCnt) % sizeof(mtTxLastVal);
	if (left != 0)
	{
		int cp = std::min(dataCnt, sizeof(mtTxLastVal) - left);
		unsigned char* src = (unsigned char*)&mtTxLastVal;
		src += left;
		memcpy(txBuffer + txBufferPos, src, cp);
		txBufferPos += cp;
		dataCnt -= cp;
	}
	while (dataCnt > 0)
	{
		mtTxLastVal = mt_rand_get_u32(&mtTx);
		unsigned int cnt = std::min(dataCnt, sizeof(mtTxLastVal));
		memcpy(txBuffer + txBufferPos, &mtTxLastVal, cnt);
		txBufferPos += cnt;
		dataCnt -= cnt;
	}

	bool rwStatus;
	rwStatus = WriteComm(comPortHandle, txBuffer, txBufferPos);
	if (rwStatus == FALSE)
	{
		LOG("Error writing to serial port\n");
	}
	else
	{
		LOG("Written: %u B [%02X]\n", txBufferPos, txBuffer[0]);
	}
	mtTxCnt += txBufferPos;
	totalTxCnt += txBufferPos;

	unsigned char rxBuffer[Settings::_UartMtTest::MAX_TX_CNT * 2];
	memset(rxBuffer, 0x66, sizeof(rxBuffer));
	DWORD numberOfBytesRead = 0;
	rwStatus = ReadComm(comPortHandle, rxBuffer, &numberOfBytesRead, sizeof(rxBuffer));
	unsigned int rxBufferPos = 0;
	if (rwStatus == FALSE)
	{
		LOG("Error reading from serial port\n");
	}
	else
	{
		if (numberOfBytesRead == 0)
		{
			LOG("%u bytes read\n", numberOfBytesRead);
		}
		else
		{
			LOG("%u bytes read [%02X]\n", numberOfBytesRead, rxBuffer[0]);
		}
		unsigned int left = numberOfBytesRead;
		unsigned int lastMismatchCount = mismatchCount;
		while (left > 0)
		{
			int mod = static_cast<unsigned int>(mtRxCnt) % sizeof(mtRxLastVal);
			if (mod == 0)
			{
				mtRxLastVal = mt_rand_get_u32(&mtRx);
				unsigned int cnt = std::min(sizeof(mtRxLastVal), left);

				for (unsigned int i=0; i<cnt; i++)
				{
                    unsigned char* ref = ((unsigned char*)&mtRxLastVal);
					if (ref[i] != rxBuffer[rxBufferPos + i])
					{
						LOG("mtRxLastVal = %08X\n", mtRxLastVal);
						LOG("ref = %02X, found = %02X\n",
							static_cast<unsigned int>(ref[i]),
							static_cast<unsigned int>(rxBuffer[rxBufferPos + i]));
						LOG("Data mismatch (1) at position %llu + %u: %02X vs %02X\n",
							mtRxCnt, i,
							static_cast<unsigned int>(ref[i]),
							static_cast<unsigned int>(rxBuffer[rxBufferPos + i]));
						struct mt_rand mtNext = mtRx;
						unsigned int mtRxNextVal = mt_rand_get_u32(&mtNext);
						LOG("mtRxNextVal = %08X\n", mtRxNextVal);
						mismatchCount++;
						break;
					}
				}
				mtRxCnt += cnt;
				rxBufferPos += cnt;
				left -= cnt;
			}
			else
			{
				unsigned int cnt = sizeof(mtRxLastVal) - mod;
				cnt = std::min(cnt, left);
				unsigned char* ref = ((unsigned char*)&mtRxLastVal) + mod;

				for (unsigned int i=0; i<cnt; i++)
				{
					if (ref[i] != rxBuffer[rxBufferPos + i])
					{
                        LOG("mtRxLastVal = %08X\n", mtRxLastVal);
						LOG("Data mismatch (2) at position %llu + %u: %02X vs %02X\n",
							mtRxCnt, i,
							static_cast<unsigned int>(ref[i]),
							static_cast<unsigned int>(rxBuffer[rxBufferPos + i]));
						struct mt_rand mtNext = mtRx;
						unsigned int mtRxNextVal = mt_rand_get_u32(&mtNext);
						LOG("mtRxNextVal = %08X\n", mtRxNextVal);
						mismatchCount++;
						break;
					}
				}
				
				mtRxCnt += cnt;
				rxBufferPos += cnt;
				left -= cnt;
			}
			
			if (lastMismatchCount != mismatchCount)
			{
				break;
			}
		}
		if (lastMismatchCount != mismatchCount)
		{
			Sleep(500);
			// restart
			PurgeComm( comPortHandle, PURGE_TXABORT | PURGE_RXABORT | PURGE_TXCLEAR | PURGE_RXCLEAR );
			Reset();
		}
	}

	UpdateTxRxDisplay();
}

void TfrmUartMtTest::UpdateTxRxDisplay(void)
{
	AnsiString mtTxCntText, mtRxCntText;
	mtTxCntText.sprintf("%llu", mtTxCnt);
	mtRxCntText.sprintf("%llu", mtRxCnt);
	edTxDataCount->Text = mtTxCntText;
	edRxDataCount->Text = mtRxCntText;
	edMismatchCount->Text = mismatchCount;
	edTotalTxDataCount->Text = totalTxCnt;	
}

void __fastcall TfrmUartMtTest::btnSingleStepTestClick(TObject *Sender)
{
	if (comPortHandle == INVALID_HANDLE_VALUE)
	{
        MessageBox(this->Handle, "Serial port is not opened!", this->Caption.c_str(), MB_ICONEXCLAMATION);
		return;
	}
	HandleTest();
}
//---------------------------------------------------------------------------



void __fastcall TfrmUartMtTest::btnResetStatsClick(TObject *Sender)
{
	mismatchCount = 0;
	totalTxCnt = 0;
	UpdateTxRxDisplay();
}
//---------------------------------------------------------------------------

void __fastcall TfrmUartMtTest::btnResetTestClick(TObject *Sender)
{
	Reset();	
}
//---------------------------------------------------------------------------

void TfrmUartMtTest::Reset(void)
{
	uint32_t seed = 189432343;
	mt_rand_init(&mtTx, seed);
	mt_rand_init(&mtRx, seed);

	mt_rand_init(&mtSize, rand());

	mtTxCnt = 0;
	mtRxCnt = 0;
	
	UpdateTxRxDisplay();
}

void __fastcall TfrmUartMtTest::ParamsChange(TObject *Sender)
{
	appSettings.UartMtTest.dataCountMin = StrToIntDef(edDataCountMin->Text, appSettings.UartMtTest.dataCountMin);
	appSettings.UartMtTest.dataCountMax = StrToIntDef(edDataCountMax->Text, appSettings.UartMtTest.dataCountMax);
	appSettings.UartMtTest.intervalMin = StrToIntDef(edIntervalMin->Text, appSettings.UartMtTest.intervalMin);
	appSettings.UartMtTest.intervalMax = StrToIntDef(edIntervalMax->Text, appSettings.UartMtTest.intervalMax);

	appSettings.UartMtTest.Validate();
}
//---------------------------------------------------------------------------

